extern crate deflate;
extern crate regex;
extern crate crc;
use std::vec::Vec;
use crc::{crc32, Hasher32};
use std::io::{self, Write};


fn ihdr_chunk(width:u32, height:u32) -> Vec<u8> {
  let mut chunk_data:Vec<u8> = Vec::with_capacity(13);
  for byte in width.to_be_bytes().iter() {
    chunk_data.push(*byte);
  }
  for byte in height.to_be_bytes().iter() {
    chunk_data.push(*byte);
  }
  chunk_data.push(8); //8 bits. Laziness.
  chunk_data.push(3); //Type 3 - Palette
  chunk_data.push(0);
  chunk_data.push(0);
  chunk_data.push(0);
  return write_general_chunk(b"IHDR".to_vec(), chunk_data);
}

fn palette_chunk() -> Vec<u8> {
  let mut chunk_data:Vec<u8> = Vec::with_capacity(6);
  let colors = [[238, 255, 125], [0, 0, 0]];
  for color in colors.iter() {
    for byte in color.iter() {
      chunk_data.push(*byte);
    }
  }
  return write_general_chunk(b"PLTE".to_vec(), chunk_data);
}

fn write_general_chunk(name:Vec<u8>, data:Vec<u8>) -> Vec<u8> {
  let mut full_chunk:Vec<u8> = Vec::with_capacity( (12 + data.len()) as usize);
  let chunk_length:u32 = data.len() as u32;
  let mut digest = crc32::Digest::new(crc32::IEEE);
  digest.write(&name);
  digest.write(&data);
  for byte in chunk_length.to_be_bytes().iter() {
    full_chunk.push(*byte);
  }
  for byte in name.iter() {
    full_chunk.push(*byte);
  }
  for byte in data.iter() {
    full_chunk.push(*byte);
  }
  for byte in digest.sum32().to_be_bytes().iter() {
    full_chunk.push(*byte);
  }
  return full_chunk;
}

fn render_png(width:u32, height:u32) -> Vec<u8> {
  use deflate::{deflate_bytes_zlib_conf, Compression};
  let mut png:Vec<u8> = Vec::with_capacity(128);
  let mut ihdr = ihdr_chunk(width, height);
  let mut plte = palette_chunk();
  let mut data:Vec<u8> = Vec::with_capacity( ((width * height) + height) as usize );
  //Signature
  png.push(137);
  png.push(80);
  png.push(78);
  png.push(71);
  png.push(13);
  png.push(10);
  png.push(26);
  png.push(10);
  png.append(&mut ihdr);
  png.append(&mut plte);
  //Create the IDAT section
  for _y in 0..height {
    data.push(0); //Scanline filter byte
    for _x in 0..width {
      data.push(0); //One byte per pixel
    }
  }
  let compressed = deflate_bytes_zlib_conf(&data, Compression::Best);
  let mut idat = write_general_chunk(b"IDAT".to_vec(), compressed);
  png.append(&mut idat);
  let mut iend = write_general_chunk(b"IEND".to_vec(), Vec::new());
  png.append(&mut iend);
  return png;
}

fn main() -> io::Result<()> {
  use std::env;
  use regex::Regex;
  let mut width:u32 = 320;
  let mut height:u32 = 320;
  let pathinfo = match env::var("PATH_INFO") {
    Ok(val) => val,
    Err(_e) => format!(""),
  };
  let re = match Regex::new( r"([\d]+)" ) {
    Ok(re) => re,
    Err(_) => panic!("Couldn't use static regex."),
  };
  
  let numbers:Vec<u32> =  re.find_iter(&pathinfo).map(|cap| cap.as_str().parse::<u32>().unwrap_or(320) ).collect();

  if numbers.len() > 0 {
    width = numbers[0];
  }

  if numbers.len() > 1 {
    height = numbers[1];
  }

  if width < 10 {
    width = 10;
  }

  if width > 4000 {
    width = 4000;
  }

  if height < 10 {
    height = 10;
  }

  if height > 4000 {
    height = 4000;
  }
  
  let png = render_png(width, height);

  println!("Status: 200 Ok");
  println!("Content-type: image/png");
  println!("X-Powered-By: Komari v0.1 (Rust)");
  println!("Content-Length: {}", png.len());
  println!("");
  let stdout = io::stdout();
  let mut handle = stdout.lock();

  handle.write_all(&png)?;

  Ok(())
}
